﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DateDiffInDays.DateDiff
{
    /// <summary>
    /// Date difference calculation by incrementing the previous date until it's equal to the subsequent date.
    /// </summary>
    class DateDiffByIncrement : IDateDiff
    {
        private int days;

        public DateDiffByIncrement(int y1, int m1, int d1, int y2, int m2, int d2) :
            base(y1, m1, d1, y2, m2, d2)
        {
            /* empty, see base constructor */
        }

        public override int GetDifference()
        {
            days = 0;
            while (DateDiffUtil.IsBeforeThan(yearPreceding, monthPreceding, dayPreceding, 
                                             yearSubsequent, monthSubsequent, daySubsequent))
            {
                days++;
                IncreasePrecedingDateByOneDay();
            }

            return days * (IsDate1AfterDate2 ? -1 : 1);
        }

        /// <summary>
        /// Increase the 'preceding date' by one day.
        /// </summary>
        private void IncreasePrecedingDateByOneDay()
        {
            if (DateDiffUtil.IsLastDayInMonth(yearPreceding, monthPreceding, dayPreceding))
            {
                if (DateDiffUtil.IsLastMonthInYear(monthPreceding))
                {
                    monthPreceding = 1;
                    dayPreceding = 1;
                    yearPreceding++;
                    Logger.GetLogger().Log("Reached year: " + yearPreceding + " - Days: " + days);
                }
                else
                {
                    dayPreceding = 1;
                    monthPreceding++;
                }
            }
            else
            {
                dayPreceding++;
            }
        }
    }
}
