﻿/*
 * Implementation of different algorithms to calculate the difference of two dates in days.
 * 
 * Algorithms:
 * - Using DateTime class of .NET framework
 * - Counting algorithm by incrementing the previous date until it's equal to the subsequent date.
 * - Direct calculating in three steps.
 * - Calculating the delay in days of both dates to an origin, then get the difference of both delays.
 * 
 * Sunshine, November 2014
 * 
 * www.sunshine2k.de || www.bastian-molkenthin.de
 */ 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using DateDiffInDays.DateDiff;

namespace DateDiffInDays
{
    public partial class MainPage : UserControl
    {
        private static readonly string STR_SEL_ALGO = "Selected algorithm: ";

        public MainPage()
        {
            InitializeComponent();

            Logger.GetLogger().SetLogBox(logBox);
        }

        private void calcButton_Click(object sender, RoutedEventArgs e)
        {
            if (!datePicker1.SelectedDate.HasValue || !datePicker2.SelectedDate.HasValue)
                return;
            
            logBox.Items.Clear();

            DateTime dt1 = datePicker1.SelectedDate.Value;
            DateTime dt2 = datePicker2.SelectedDate.Value;

            IDateDiff dateAlgo = getSelectedAlgo(dt1, dt2);
                        
            resultTextBox.Text = dateAlgo.GetDifference().ToString();
        }

        private IDateDiff getSelectedAlgo(DateTime dt1, DateTime dt2)
        {
            if (algoIncrementRadioButton.IsChecked.Value)
            {
                Logger.GetLogger().Log(STR_SEL_ALGO + "Increment Algorithm");
                return new DateDiffByIncrement(dt1.Year, dt1.Month, dt1.Day, dt2.Year, dt2.Month, dt2.Day);
            } 
            else if (algoThreeStepsRadioButton.IsChecked.Value)
            {
                Logger.GetLogger().Log(STR_SEL_ALGO + "Three steps Algorithm");
                return new DateDiffDirectThreeSteps(dt1.Year, dt1.Month, dt1.Day, dt2.Year, dt2.Month, dt2.Day);
            }
            else if (algoOriginBasedRadioButton.IsChecked.Value)
            {
                Logger.GetLogger().Log(STR_SEL_ALGO + "Origin-based Algorithm");
                return new DateDiffByDiffToOrigin(dt1.Year, dt1.Month, dt1.Day, dt2.Year, dt2.Month, dt2.Day);
            }
            else if (algoDotnetRadioButton.IsChecked.Value)
            {
                Logger.GetLogger().Log(STR_SEL_ALGO + ".NET Date class Algorithm");
                return new DateDiffDotnetLib(dt1.Year, dt1.Month, dt1.Day, dt2.Year, dt2.Month, dt2.Day);
            }
            else
            {
                throw new InvalidOperationException("No algorithm selected");
            }
        }

    }
}
